/*
 *   Copyright 2017-2020, Schlumberger
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 */

package org.opengroup.osdu.notification.service;

import org.opengroup.osdu.core.common.http.HttpClient;
import org.opengroup.osdu.core.common.http.HttpRequest;
import org.opengroup.osdu.core.common.http.HttpResponse;
import org.opengroup.osdu.core.common.logging.JaxRsDpsLog;
import org.opengroup.osdu.core.common.model.http.DpsHeaders;
import org.opengroup.osdu.core.common.model.notification.*;
import org.opengroup.osdu.notification.auth.factory.AuthFactory;
import org.opengroup.osdu.notification.auth.interfaces.SecretAuth;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.web.context.annotation.RequestScope;

import java.util.HashMap;
import java.util.Map;

@Component
public class NotificationHandler {
    private final static Logger LOGGER = LoggerFactory.getLogger(NotificationHandler.class);
    @Autowired
    private HttpClient httpClient;
    @Autowired
    private SubscriptionHandler subscriptionHandler;
    @Autowired
    private AuthFactory authFactory;
    @Value("${app.waitingTime:30000}")
    private int WAITING_TIME;

    public HttpResponse notifySubscriber(String notificationId, String pubsubMessage, Map<String, String> headerAttributes) throws Exception {
        Subscription subscription = subscriptionHandler.getSubscriptionFromCache(notificationId);
        Secret secret = subscription.getSecret();
        String endpoint = subscription.getPushEndpoint();
        String secretType = secret.getSecretType();
        String pushUrl = "";
        Map<String, String> requestHeader = new HashMap<String, String>();

        // Authentication Secret
        SecretAuth secretAuth = authFactory.getSecretAuth(secretType);
        secretAuth.setSecret(secret);
        pushUrl = secretAuth.getPushUrl(endpoint);
        requestHeader = secretAuth.getRequestHeaders();

        requestHeader.put(DpsHeaders.CONTENT_TYPE, "application/json");
        requestHeader.put(DpsHeaders.CORRELATION_ID, headerAttributes.get(DpsHeaders.CORRELATION_ID));
        requestHeader.put(DpsHeaders.DATA_PARTITION_ID, headerAttributes.get(DpsHeaders.DATA_PARTITION_ID));

        HttpRequest request = HttpRequest.post().url(pushUrl).headers(requestHeader).body(pubsubMessage).connectionTimeout(WAITING_TIME).build();
        HttpResponse response = httpClient.send(request);
        this.LOGGER.debug("Sending out notification to endpoint: " + endpoint);
        return response;
    }
}
